# Copyright (C) 2016 resonic.ru
#
# File: UVLExport.py
# Last Modified: 01.07.2016

import subprocess, os
import c4d
from c4d import gui, storage
from c4d import documents, plugins

GROUP_ID1 = 1000
GROUP_ID2 = 1001
GROUP_ID3 = 1002
GROUP_ID4 = 1003

TEXTBOX = 1004
TEXTSTATIC = 1005

BUTTON1 = 1011
BUTTON2 = 1012
BUTTON3 = 1013
BUTTON4 = 1014

COMBO_TYPE = 1020
COMBO_UVS = 1021
SUB_SUBD = 1022
SUB_POLY = 1023
SUB_EDIT = 1024
SUB_NEW = 1025

CHK_WELD = 1030
CHK_CLEAN = 1031
CHK_DETACH = 1032
CHK_GEOM = 1033
CHK_DROP = 1034

SYS_SILENT = 1040

tmp_folder = os.path.join(storage.GeGetC4DPath(c4d.C4D_PATH_PREFS), "uvlcache")
file_path = ""
args = []


class UVLExport(gui.GeDialog):
    def CreateLayout(self):
        self.GroupBegin(GROUP_ID1, c4d.BFH_SCALEFIT, 3, 1)
        self.GroupBorderNoTitle(c4d.BORDER_ROUND)
        self.GroupBorderSpace(10, 10, 10, 10)
        self.AddStaticText(TEXTSTATIC, c4d.BFH_LEFT, 80, 10, "UVL Path: ", 0)
        self.AddEditText(TEXTBOX, c4d.BFH_SCALEFIT, 50, 10)
        self.AddButton(BUTTON4, c4d.BFH_RIGHT, name="...")
        self.GroupEnd()
        #
        self.GroupBegin(GROUP_ID2, c4d.BFH_SCALEFIT, 3, 1)
        self.GroupBorderNoTitle(c4d.BORDER_ROUND)
        self.GroupBorderSpace(10, 10, 10, 10)
        self.AddComboBox(COMBO_TYPE, c4d.BFH_SCALEFIT, 1, 12)
        self.AddChild(COMBO_TYPE, SUB_SUBD, "SUBD")
        self.AddChild(COMBO_TYPE, SUB_POLY, "POLY")
        self.AddComboBox(COMBO_UVS, c4d.BFH_SCALEFIT, 1, 12)
        self.AddChild(COMBO_UVS, SUB_EDIT, "EDIT")
        self.AddChild(COMBO_UVS, SUB_NEW, "NEW")
        self.GroupEnd()
        #
        self.GroupBegin(GROUP_ID3, c4d.BFH_SCALEFIT, 3, 1)
        self.GroupBorderNoTitle(c4d.BORDER_ROUND)
        self.GroupBorderSpace(10, 10, 10, 10)
        self.AddCheckbox(CHK_WELD, c4d.BFH_SCALEFIT, 1, 1, name="WELD")
        self.AddCheckbox(CHK_CLEAN, c4d.BFH_SCALEFIT, 1, 1, name="CLEAN")
        self.AddCheckbox(CHK_DETACH, c4d.BFH_SCALEFIT, 1, 1, name="DETACH")
        self.AddCheckbox(CHK_GEOM, c4d.BFH_SCALEFIT, 1, 1, name="GEOM")
        self.AddCheckbox(CHK_DROP, c4d.BFH_SCALEFIT, 1, 1, name="DROP")
        self.AddCheckbox(SYS_SILENT, c4d.BFH_SCALEFIT, 1, 1, name="SILENT")
        self.GroupEnd()
        #
        self.GroupBegin(GROUP_ID3, c4d.BFH_SCALEFIT, 3, 1)
        self.GroupBorderNoTitle(c4d.BORDER_ROUND)
        self.GroupBorderSpace(10, 10, 10, 10)
        self.AddButton(BUTTON1, c4d.BFH_SCALEFIT, name="Exchange")
        self.AddButton(BUTTON2, c4d.BFH_SCALEFIT, name="Update")
        self.AddButton(BUTTON3, c4d.BFH_SCALEFIT, name="Close")
        self.GroupEnd()
        return True

    def InitValues(self):
        self.SetString(TEXTBOX, Reader(tmp_folder))
        self.SetInt32(COMBO_TYPE, 1022)
        self.SetInt32(COMBO_UVS, 1024)
        self.SetBool(SYS_SILENT, True)
        self.Enable(BUTTON2, False)
        return True

    def Command(self, id, msg):
        if id == BUTTON1:
            if self.GetInt32(COMBO_TYPE) == 1022:
                args.append("SUBD")
            if self.GetInt32(COMBO_TYPE) == 1023:
                args.append("Poly")
            if self.GetInt32(COMBO_UVS) == 1024:
                args.append("Edit")
            if self.GetInt32(COMBO_UVS) == 1025:
                args.append("New")
            if self.GetBool(CHK_WELD):
                args.append("Weld")
            if self.GetBool(CHK_CLEAN):
                args.append("Clean")
            if self.GetBool(CHK_DETACH):
                args.append("Detach")
            if self.GetBool(CHK_GEOM):
                args.append("Geom")
            if self.GetBool(CHK_DROP):
                args.append("Drop")

            if not doc.GetActiveObjects(c4d.GETACTIVEOBJECTFLAGS_CHILDREN):
                gui.MessageDialog("Select the objects!")
            else:
                KillTask()
                uvl_path = self.GetString(TEXTBOX)
                OBJExporter()
                if file_path:
                    try:
                        subprocess.Popen([uvl_path, file_path, "-plugin," + ','.join(args)])
                    except:
                        gui.MessageDialog("uvlayout.exe not found")
                    del args[:]
                    self.Enable(BUTTON2, True)
                    self.SetString(BUTTON1, "Reload")
                    self.SetTimer(1500)

        elif id == BUTTON2:
            OBJExporter()
            self.SetTimer(1500)

        elif id == BUTTON3:
            CacheDel(tmp_folder)
            KillTask()
            Reader(tmp_folder, 1)
            self.Close()
            
        elif id == BUTTON4:
            uvl_path = c4d.storage.LoadDialog(type=c4d.FILESELECTTYPE_ANYTHING, title="UVLayout.exe", 
                                        flags=c4d.FILESELECT_LOAD)
            self.SetString(TEXTBOX,uvl_path)
            Reader(tmp_folder, 1)
            
        return True

    def Timer(self, msg):
        out_file = file_path[:-4] + ".out"
        tmp_obj = file_path[:-4] + "_.obj"
        if os.path.exists(out_file):
            try:
                os.remove(tmp_obj)
            except:
                pass
            os.rename(out_file, tmp_obj)
            OBJImporter(tmp_obj)
            print "Loaded!"


def Reader(path, mode=0):
    cfg_path = path + r"\config\config.txt"
    if not os.path.exists(path):        
        os.mkdir(path)
    if not os.path.exists(path + r"\config"):
        os.mkdir(path + r"\config")        
    if not os.path.exists(cfg_path) or mode == 1:
        f = open(cfg_path, 'w')
        if mode == 0:
            f.write("C:\Program Files (x86)\headus UVLayout v2 Professional\uvlayout.exe")
        else:
            f.write(dlg.GetString(TEXTBOX))
        f.close()
    if os.path.exists(cfg_path):
        tmp_file = open(cfg_path, 'r')
        txt = tmp_file.read()  # .split()
        tmp_file.close()
        return txt


def CacheDel(path):
    subprocess.Popen('cmd /c del /f /q ' + '"' + path + '"', shell=False)


def KillTask():
    subprocess.Popen('taskkill /f /im cyeatP.exe', shell=False)
    p = subprocess.Popen('taskkill /f /im cysliceP.exe', shell=False)
    p.wait()


def OBJExporter():
    objects = doc.GetActiveObjects(c4d.GETACTIVEOBJECTFLAGS_CHILDREN)
    doc_temp = c4d.documents.IsolateObjects(doc, objects)
    if doc_temp is None:
        return

    plug = plugins.FindPlugin(1030178, c4d.PLUGINTYPE_SCENESAVER)
    if plug is None:
        return

    global file_path
    if dlg.GetBool(SYS_SILENT):
        if not os.path.exists(tmp_folder):
            os.mkdir(tmp_folder)
        file_path = tmp_folder + r"\swp.obj"
    else:
        file_path = c4d.storage.LoadDialog(title="Save File for OBJ Export", flags=c4d.FILESELECT_SAVE,
                                           force_suffix="obj")
        if not file_path:
            print "Cancel"
            return

    op = {}
    if plug.Message(c4d.MSG_RETRIEVEPRIVATEDATA, op):
        if "imexporter" not in op: return
        obj_export = op["imexporter"]
        if obj_export is None:
            return

        unit_scale = c4d.UnitScaleData()
        unit_scale.SetUnitScale(1.0, c4d.DOCUMENT_UNIT_CM)
        obj_export[c4d.OBJEXPORTOPTIONS_SCALE] = unit_scale

        obj_export[c4d.OBJEXPORTOPTIONS_TEXTURECOORDINATES] = True
        obj_export[c4d.OBJEXPORTOPTIONS_MATERIAL] = c4d.OBJEXPORTOPTIONS_MATERIAL_MATERIAL
        obj_export[c4d.OBJEXPORTOPTIONS_OBJECTS_AS_GROUPS] = True

        obj_export[c4d.OBJEXPORTOPTIONS_POINTTRANSFORM_FLIPX] = False
        obj_export[c4d.OBJEXPORTOPTIONS_POINTTRANSFORM_FLIPY] = False
        obj_export[c4d.OBJEXPORTOPTIONS_POINTTRANSFORM_FLIPZ] = False

        obj_export[c4d.OBJEXPORTOPTIONS_POINTTRANSFORM_SWAPYZ] = False
        obj_export[c4d.OBJEXPORTOPTIONS_POINTTRANSFORM_SWAPXZ] = False
        obj_export[c4d.OBJEXPORTOPTIONS_POINTTRANSFORM_SWAPXY] = False

        if c4d.documents.SaveDocument(doc_temp, file_path, c4d.SAVEDOCUMENTFLAGS_DONTADDTORECENTLIST, 1030178):
            print "Document successfully exported to:" + file_path
        else:
            print "Export failed!"
    c4d.StatusClear()


def OBJImporter(obj_path):
    if obj_path is None: return
    tmp_obj = doc.GetObjects()
    for i in tmp_obj:
        i[c4d.ID_BASEOBJECT_VISIBILITY_EDITOR] = True
        i[c4d.ID_BASEOBJECT_VISIBILITY_RENDER] = True
    plug = plugins.FindPlugin(1030177, c4d.PLUGINTYPE_SCENELOADER)
    if plug is None: return

    op = {}
    if plug.Message(c4d.MSG_RETRIEVEPRIVATEDATA, op):
        if "imexporter" not in op: return
        obj_import = op["imexporter"]
        if obj_import is None: return

        unit_scale = c4d.UnitScaleData()
        unit_scale.SetUnitScale(1.0, c4d.DOCUMENT_UNIT_CM)
        obj_import[c4d.OBJIMPORTOPTIONS_SCALE] = unit_scale

        obj_import[c4d.OBJIMPORTOPTIONS_PHONG_ANGLE_DEFAULT] = 22.5
        obj_import[c4d.OBJIMPORTOPTIONS_TEXTURECOORDINATES] = True
        obj_import[c4d.OBJIMPORTOPTIONS_SPLITBY] = c4d.OBJIMPORTOPTIONS_SPLITBY_OBJECT
        obj_import[c4d.OBJIMPORTOPTIONS_MATERIAL] = c4d.OBJIMPORTOPTIONS_MATERIAL_MTLFILE

        obj_import[c4d.OBJIMPORTOPTIONS_POINTTRANSFORM_FLIPX] = False
        obj_import[c4d.OBJIMPORTOPTIONS_POINTTRANSFORM_FLIPY] = False
        obj_import[c4d.OBJIMPORTOPTIONS_POINTTRANSFORM_FLIPZ] = False

        obj_import[c4d.OBJIMPORTOPTIONS_POINTTRANSFORM_SWAPXY] = False
        obj_import[c4d.OBJIMPORTOPTIONS_POINTTRANSFORM_SWAPXZ] = False
        obj_import[c4d.OBJIMPORTOPTIONS_POINTTRANSFORM_SWAPYZ] = False

        c4d.documents.MergeDocument(doc, obj_path, c4d.SCENEFILTER_OBJECTS | c4d.SCENEFILTER_MATERIALS, None)


if __name__ == '__main__':
    dlg = UVLExport()
    dlg.Open(c4d.DLG_TYPE_ASYNC, xpos=-1, ypos=-1, defaultw=250, defaulth=200)
